/*
 * ithildin.h: standard include file used project-wide
 * 
 * Copyright 2002-2006 the Ithildin Project.
 * See the COPYING file for more information on licensing and use.
 * 
 * $Id$
 */

#ifndef STAND_H
#define STAND_H

#include <ithildin/config.h>
#ifdef MODULE
/* Include these if it is a module build.  modconfig.h must be supplied by
 * the module, it is intended to provide overrides to config.h */
# include "modconfig.h"
#endif

/* fix for idiot Linux breakage where we have to define '_GNU_SOURCE' to get
 * RTLD_NEXT/RTLD_DEFAULT (and god knows what else) even though they are
 * explicitly specified in the SysV dl*() specs and LOTS OF NON GNU SYSTEMS USE
 * THEM. */
#ifdef HAVE_FEATURES_H
# define _GNU_SOURCE
#endif

/* global config files are included here */
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <math.h>
#ifdef HAVE_MEMORY_H
# include <memory.h>
#endif
#ifdef POLLER_POLL
# include <poll.h>
#endif
#include <signal.h>
#include <stdarg.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
# include <strings.h>
#endif
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#ifdef HAVE_SYS_MMAN_H
# include <sys/mman.h>
#endif
#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif
#ifdef POLLER_KQUEUE
# include <sys/event.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
# include <sys/socket.h>
#endif
#ifdef HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif
#ifdef HAVE_SYS_TIME_H
# include <sys/time.h>
#endif
#ifdef HAVE_TIME_H
# include <time.h>
#endif

#ifdef HAVE_MACHINE_ENDIAN_H
# include <machine/endian.h>
#elif defined(WORDS_BIGENDIAN)
# if !defined(BIG_ENDIAN)
#  define BIG_ENDIAN 4321
# endif
# undef BYTE_ORDER
# define BYTE_ORDER BIG_ENDIAN
#endif

#ifdef HAVE_NETDB_H
# include <netdb.h>
#endif
#ifdef HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif
#ifdef HAVE_ARPA_INET_H
# include <arpa/inet.h>
#endif

/* many files depend on queue.h, put it at the top */
#include <ithildin/queue.h>

/* Handy stuff for handling printf-like functions and warnings in gcc (thanks
 * hop & larne @ EFnet) */
/* some useful gcc-only defines to tag purposefully unused variables, and to
 * tag functions as being printf-like */
#if defined(__GNUC__)
# define __UNUSED __attribute__ ((__unused__))
# define __PRINTF(x) __attribute__ ((format (printf, x, x + 1)))
# ifdef __FreeBSD__
#  define __PRINTF0(x) __attribute__ ((format (printf0, x, x + 1)))
# else
#  define __PRINTF0(x) __attribute__ ((format (printf, x, x + 1)))
# endif
#else
# define __UNUSED
# define __PRINTF(x)
# define __PRINTF0(x)
#endif
#define IDSTRING(var,string) static const char var[] __UNUSED = string

/* if we're debugging, don't inline stuff! */
#ifdef DEBUG
# undef inline
# define inline
#endif

#include <ithildin/callback.h>
#include <ithildin/conf.h>
#include <ithildin/event.h>
#include <ithildin/global.h>
#include <ithildin/hash.h>
#include <ithildin/log.h>
#include <ithildin/malloc.h>
#include <ithildin/md5.h>
#include <ithildin/module.h>
#include <ithildin/socket.h>
#include <ithildin/string.h>
#include <ithildin/timer.h>
#include <ithildin/util.h>

/* keep assert at the bottom in case any other headers get 'cute' */
#ifndef DEBUG
# define NDEBUG
#endif
#ifdef HAVE_ASSERT_H
# include <assert.h>
#endif

/* keep dmalloc at the bottom of the file to override any macros */
#ifdef USE_DMALLOC
# include <dmalloc.h>
# ifdef DEBUG
#  define DMALLOC_FUNC_CHECK
# endif
#endif

#endif
/* vi:set ts=8 sts=4 sw=4 tw=76 et: */
