/*
 * malloc.h: malloc support prototypes
 * 
 * Copyright 2003 the Ithildin Project.
 * See the COPYING file for more information on licensing and use.
 * 
 * $Id$
 */

#ifndef MALLOC_H
#define MALLOC_H

#if !defined(USE_DMALLOC) && defined(MALLOC_OVERRIDE)
# define calloc ith_calloc
# define free ith_free
# define malloc ith_malloc
# define realloc ith_realloc
/* the actual functions.. */
void *ith_calloc(size_t, size_t);
void ith_free(void *);
void *ith_malloc(size_t);
void *ith_realloc(void *, size_t);
#endif

extern struct _malloc_statistics {
    uint64_t    malloc_calls;       /* Number of calls to malloc/calloc */
    uint64_t    realloc_calls;      /* Number of callcs to realloc */
    uint64_t    free_calls;         /* Number of calls to free() */

    uint64_t    heaps;              /* Number of 'performance managed' heaps */

    uint64_t    palloc_calls;       /* Number of calls to page allocator */
    uint64_t    pfree_calls;        /* Number of calls to page freer */
    
    size_t      pagesize;           /* Size (in bytes) of allocated blocks */
    uint64_t    pages_inuse;        /* Current count of blocks in-use */
} malloc_statistics;

/* Use the MALLOC_STATISTICS macro to easily test whether or not allocator
 * statistics are available for consumption.  The thing it is using to
 * determine this may change over time. */
#define MALLOC_STATISTICS malloc_statistics.pagesize

#endif

/* vi:set ts=8 sts=4 sw=4 tw=76 et: */
