/*
 * string.h: prototypes for string-handling functions.
 * 
 * Copyright 2002 the Ithildin Project.
 * See the COPYING file for more information on licensing and use.
 * 
 * $Id$
 */

#ifndef ISTRING_H
#define ISTRING_H

/* these are tables for converting strings to ASCII upper/lowercase. */
extern unsigned char tolowertab[256];
#undef tolower
#define tolower(c) tolowertab[(unsigned char)(c)]
extern unsigned char touppertab[256];
#undef toupper
#define toupper(c) touppertab[(unsigned char)(c)]

/* matching functions */
int match(const char *wild, const char *string);
int hostmatch(const char *wild, const char *string);
int ipmatch(const char *wild, const char *string);

/* string conversion functions */
bool str_conv_bool(const char *, bool);
const char *bool_conv_str(bool, const char *, const char *);
time_t str_conv_time(const char *, int);
char *time_conv_str(time_t);
int str_conv_int(const char *, int);
char *int_conv_str(int);

/* baseX encoding/decoding/conversion functions.  please note that, in the case
 * of base64, it is *NOT* compatible with RFC1521! */
#define BASE2_ENCODING      1
#define BASE4_ENCODING      2
#define BASE8_ENCODING      3
#define BASE16_ENCODING     4
#define BASE32_ENCODING     5
#define BASE64_ENCODING     6
size_t str_base_encode(char, char *, char *, size_t);
size_t str_base_decode(char, char *, char *, size_t);

/* fmtcheck */
#ifndef HAVE_FMTCHECK
const char *fmtcheck(const char *, const char *);
#endif
#ifndef HAVE_STRLCAT
size_t strlcat(char *dst, const char *src, size_t siz);
#endif
#ifndef HAVE_STRLCPY
size_t strlcpy(char *dst, const char *src, size_t siz);
#endif
#ifndef HAVE_STRSEP
char *strsep(char **from, const char *delim);
#endif


#endif
/* vi:set ts=8 sts=4 sw=4 tw=76 et: */
