/*
 * strsep.c: String separation routine
 * 
 * Copyright 2005 the Ithildin Project.
 * See the COPYING file for more information on licensing and use.
 * 
 * this file contains a variety of string manipulation functions.  some of them
 * are libc replacements which were designed to be as fast as possible for the
 * intended purposes of the software, there are also a load of others which
 * serve many varied purposes.
 */

/* this function was (sort of) borrowed from FreeBSD 4's libc.  I've shuffled a
 * bit around and tried to make it more clear what's going on.  The general
 * idea is that you can call strsep() with the address of a string to work on,
 * and a list of delimiting characters.  it will break up the string as it is
 * called, returning one argument at a time.  it may return an empty argument
 * which can be detected by checking to see if the return value starts with a
 * \0 character. */
char *strsep(char **from, const char *delim) {
    char *s, *dp, *ret;

    if ((s = *from) == NULL)
        return NULL;

    ret = s;
    while (*s != '\0') {
        /* loop until the end of s, checking against each delimiting character,
         * if we find a delimiter set **s to '\0' and return our previous token
         * to the user. */
        dp = (char *)delim;
        while (*dp != '\0') {
            if (*s == *dp) {
                *s = '\0';
                *from = s + 1;
                return ret;
            }
            dp++;
        }
        s++;
    }
    /* end of string case */
    *from = NULL;
    return ret;
}

