/*
 * registered.c: Flag mode for registered channels.
 * 
 * Copyright 2002-2007 the Ithildin Project.
 * See the COPYING file for more information on licensing and use.
 * 
 * This module simply adds a flag mode for channels to indicate that they are
 * registered.
 */

#include <ithildin/ithildin.h>

#include "ircd.h"

IDSTRING(rcsid, "$Id$");

MODULE_REGISTER("$Rev$");
const char *mdepends[] = MDEPENDS;

unsigned char chanmode_registered;

CHANMODE_FUNC(chanmode_registered_handler);

MODULE_LOADER(registered) {

    if (!get_module_savedata(savelist, "chanmode_registered",
                &chanmode_registered))
        chanmode_request('r', &chanmode_registered, CHANMODE_FL_D,
                chanmode_registered_handler, chanmode_flag_query, 0, NULL);
    else
        chanmode_update_funcs(chanmode_registered,
                chanmode_registered_handler, chanmode_flag_query);

#define ERR_ONLYSERVERSCANCHANGE 468
    CMSG("468", "%s :Only servers can change that mode.");

    return 1;
}

MODULE_UNLOADER(registered) {
    
    if (reload)
        add_module_savedata(savelist, "chanmode_registered",
                sizeof(chanmode_registered), &chanmode_registered);
    else
        chanmode_release(chanmode_registered);

    DMSG(ERR_ONLYSERVERSCANCHANGE);
}

/* Handle the 'r' channel mode.  Make sure the right folks are the only ones
 * who can set it. */
CHANMODE_FUNC(chanmode_registered_handler) {

    *argused = 0;
    if (cli != NULL && !CLIENT_MASTER(cli)) {
        sendto_one(cli, RPL_FMT(cli, ERR_ONLYSERVERSCANCHANGE), chan->name);
        return CHANMODE_FAIL; /* nyet comrade */
    }
    
    switch (set) {
        case CHANMODE_SET:
            chanmode_setflag(chan, mode);
            break;
            /* fallthrough to the unset case */
        case CHANMODE_UNSET:
            chanmode_unsetflag(chan, mode);
            break;
    }

    return CHANMODE_OK;
}

/* vi:set ts=8 sts=4 sw=4 tw=76 et: */
