#define RFC1459_PKT_LEN 512
#define RFC1459_MAXARGS 15

#ifndef MAX_PACKET_LEN
# define MAX_PACKET_LEN RFC1459_PKT_LEN
#endif

#ifndef MAX_COMMAND_ARGS
# define MAX_COMMAND_ARGS RFC1459_MAXARGS
#endif

const uint64_t protocol_buffer_size = MAX_PACKET_LEN;

HOOK_FUNCTION(input);
struct send_msg *output(struct protocol_sender *, char *, char *, char *,
        va_list);

/* input as much data as we can from the user. */
HOOK_FUNCTION(input) {
    isocket_t *sp = (isocket_t *)data;
    connection_t *cp = (connection_t *)sp->udata;
    int ret = 0;
    char *s;

    /* If we were force-called (ep is NULL) we assume there is data in our
     * buffer.  Jump into the loop. */
    if ((ep == NULL && cp->buflen != 0) || (cp->buflen == cp->bufsize))
        goto loop_entrance;

    while ((ret = socket_read(sp, cp->buf + cp->buflen,
                    cp->bufsize - cp->buflen)) > 0) {
loop_entrance:
        cp->buflen += ret;
        cp->stats.recv += ret;

        while (cp->buflen > 0) {
            /* if the buffer is full but no terminating character (\n) is
               found, we cheat */
            if (cp->buflen == cp->bufsize) {
                /* actually, we might have lots of commands in here, try to
                 * find a \n without overruning the buffer.  If we do, sally
                 * forth undeterred, if we don't, trim the message */
                s = cp->buf;
                while (*s != '\n' && s < cp->buf + cp->bufsize)
                    s++;

                if (s == cp->buf + cp->bufsize) {
                    cp->buf[cp->bufsize - 1] = '\n';
                    s = cp->buf + cp->bufsize - 1;
                }
            } else {
                cp->buf[cp->buflen] = '\0';
                s = strchr(cp->buf, '\n');
                if (s == NULL)
                    break; /* no separator found, try reading some more */
            }
            /* make a local copy of the packet, then pass it off to the parser. 
             * Bump the rest of the contents of 'buf' down if we read more than
             * we have parsed. */
            /* null-terminate and get rid of the [\r]\n sequence. */
            if (*(s - 1) == '\r')
                *(s - 1) = '\0';
            else
                *s = '\0';
            log_debug("[%s] <%s< %s", ircd.me->name,
                    (cp->cli != NULL ? cp->cli->nick :
                     (cp->srv != NULL ? cp->srv->name : "")), cp->buf);
            if ((ret = packet_parse(cp)) == IRCD_CONNECTION_CLOSED)
                return (void *)ret; /* connection closed, stop immediately */
            s++; /* increment s, now we see if our packet contains more data */
            if (s - cp->buf < cp->buflen) {
                cp->buflen -= s - cp->buf;
                memmove(cp->buf, s, cp->buflen);
                cp->buf[cp->buflen] = '\0';
            } else {
                cp->buflen = 0; /* all done here! */
                *cp->buf = '\0'; /* zero it out */
            }

            if (ret == IRCD_PROTOCOL_CHANGED)
                return (void *)ret;
        }
    }

    /* if there are any errors they will be picked up elsewhere */
    return (void *)0;
}

/* define RFC1459_SEND_MSG_LONG to use the long (nick!user@host) output
 * format. */
struct send_msg *output(struct protocol_sender *from, char *cmd, char *to,
        char *msg, va_list args) {
    static char buf[MAX_PACKET_LEN];
    static struct send_msg sm = {buf, 0};

    if (to != NULL && *to == '\0') /* handle numerics for unreged clients */
        to = "*";

#ifdef RFC1459_SEND_MSG_LONG
    if (from->client != NULL)
        sm.len = sprintf(buf, ":%s!%s@%s %s %s%s", from->client->nick,
                from->client->user, from->client->host, cmd,
                (to != NULL ? to : ""), (to != NULL ? " " : ""));
#else
    if (from->client != NULL)
        sm.len = sprintf(buf, ":%s %s %s%s", from->client->nick, cmd,
                (to != NULL ? to : ""), (to != NULL ? " " : ""));
#endif
    else if (from->server != NULL)
        sm.len = sprintf(buf, ":%s %s %s%s", from->server->name, cmd, 
                (to != NULL ? to : ""), (to != NULL ? " " : ""));
    else
        sm.len = sprintf(buf, "%s %s%s", cmd,
                (to != NULL ? to : ""), (to != NULL ? " " : ""));
    if (msg != NULL)
        sm.len += vsnprintf(&buf[sm.len], MAX_PACKET_LEN - 3 - sm.len, msg,
                args);

    /* XXX: GREAT POTENTIAL FOR EVIL.  Someone running their server in
     * debugmode can see the sent traffic from any server.  Evil, evil,
     * evil! */
    log_debug("[%s] >%s> %s", (from->client != NULL ? from->client->nick :
                (from->server != NULL ? from->server->name : "")), 
            (to != NULL ? to : ""), buf);

    strcpy(&buf[sm.len], "\r\n");
    sm.len += 2;

    return &sm;
}

