/*
 * registered.c: Flag mode for registered nicknames.
 * 
 * Copyright 2002-2007 the Ithildin Project.
 * See the COPYING file for more information on licensing and use.
 * 
 * This module simply adds a flag mode for nicknames to indicate that they are
 * registered.
 */

#include <ithildin/ithildin.h>

#include "ircd.h"
#include "usermodes/registered.h"
#include "commands/mode.h"
#include "commands/whois.h"

IDSTRING(rcsid, "$Id$");

MODULE_REGISTER("$Rev$");
const char *mdepends[] = MDEPENDS;

unsigned char usermode_registered;

USERMODE_FUNC(usermode_registered_handler);
HOOK_FUNCTION(usermode_registered_nick_hook);
HOOK_FUNCTION(usermode_registered_whois_hook);

MODULE_LOADER(umode_reg) {

    if (!get_module_savedata(savelist, "usermode_registered",
                &usermode_registered))
        usermode_request('r', &usermode_registered, USERMODE_FL_GLOBAL, -1,
                usermode_registered_handler);
    else
        usermode_update_func(usermode_registered,
                usermode_registered_handler);

    add_hook(ircd.events.client_nick, usermode_registered_nick_hook);
    add_hook(whois_event, usermode_registered_whois_hook);

#define RPL_WHOISREGNICK 307
    CMSG("307", "%s :has identified for this nick");

    return 1;
}

MODULE_UNLOADER(umode_reg) {
    
    if (reload)
        add_module_savedata(savelist, "usermode_registered",
                sizeof(usermode_registered), &usermode_registered);
    else
        usermode_release(usermode_registered);

    remove_hook(ircd.events.client_nick, usermode_registered_nick_hook);
    remove_hook(whois_event, usermode_registered_whois_hook);

    DMSG(RPL_WHOISREGNICK);
}

/* Don't let users change their registered/non-registered setting. */
USERMODE_FUNC(usermode_registered_handler) {
    /* clients cannot modify the +r mode themselves.  note however that
     * this will *not* cause the mode to fail for remote clients
     * because of the test in usermode_set() so a local client check
     * here is unnecessary. */
    if (by == cli && !CLIENT_MASTER(cli))
        return 0;

    return 1; /* otherwise, s'alright! */
}

HOOK_FUNCTION(usermode_registered_nick_hook) {
    client_t *cli = (client_t *)data;
    char *fakeargv[1];
    char xxx[2]; /* XXX: bad hack :) */

    if (ISREGNICK(cli)) {
        fakeargv[0] = xxx;
        xxx[0] = usermode_registered;
        xxx[1] = '\0';
        user_mode(cli, cli, 1, fakeargv, 1);
    }

    return HOOK_COND_NEUTRAL;
}

HOOK_FUNCTION(usermode_registered_whois_hook) {
    client_t *target = (client_t *)data;
    client_t *cli = cptr.cli;

    if (ISREGNICK(target))
        sendto_one(cli, RPL_FMT(cli, RPL_WHOISREGNICK), target->nick);

    return NULL;
}

/* vi:set ts=8 sts=4 sw=4 tw=76 et: */
